@@ -1,21 +1,47 @@
 """Python module for accessing SkyQ box and EPG, and sending commands."""
-import time
+import importlib
+import json
+import logging
 import math
 import socket
-import requests
-import json
-import xmltodict
-import logging
-import importlib
-import pycountry
+import time
 from datetime import datetime, timedelta
 from http import HTTPStatus
 from operator import attrgetter
 
+import pycountry
+import requests
 import websocket
+import xmltodict
 
+from .classes.channel import Channel
+from .classes.channelepg import ChannelEPG
+from .classes.channellist import ChannelList
+from .classes.device import Device
+from .classes.media import Media
+from .classes.programme import Programme, RecordedProgramme
 from .const import (
+    APP_EPG,
+    APP_STATUS_VISIBLE,
+    COMMANDS,
+    CONNECTTIMEOUT,
+    CURRENT_TRANSPORT_STATE,
+    CURRENT_URI,
+    EPG_ERROR_NO_DATA,
+    EPG_ERROR_PAST_END,
+    KNOWN_COUNTRIES,
+    PVR,
+    REST_BASE_URL,
+    REST_CHANNEL_LIST,
+    REST_PATH_DEVICEINFO,
+    REST_PATH_SYSTEMINFO,
+    REST_RECORDING_DETAILS,
     SKY_PLAY_URN,
+    SKY_STATE_OFF,
+    SKY_STATE_ON,
+    SKY_STATE_PAUSED,
+    SKY_STATE_PLAYING,
+    SKY_STATE_STANDBY,
     SKYCONTROL,
     SOAP_ACTION,
     SOAP_CONTROL_BASE_URL,
@@ -23,39 +49,14 @@ from .const import (
     SOAP_PAYLOAD,
     SOAP_RESPONSE,
     SOAP_USER_AGENT,
+    TIMEOUT,
     UPNP_GET_MEDIA_INFO,
     UPNP_GET_TRANSPORT_INFO,
     WS_BASE_URL,
     WS_CURRENT_APPS,
-    REST_BASE_URL,
-    REST_CHANNEL_LIST,
-    REST_RECORDING_DETAILS,
-    REST_PATH_SYSTEMINFO,
-    REST_PATH_DEVICEINFO,
-    CURRENT_URI,
-    CURRENT_TRANSPORT_STATE,
-    APP_STATUS_VISIBLE,
-    PVR,
     XSI,
-    PAST_END_OF_EPG,
-    CONNECTTIMEOUT,
-    TIMEOUT,
-    COMMANDS,
-    SKY_STATE_PLAYING,
-    SKY_STATE_PAUSED,
-    SKY_STATE_STANDBY,
-    SKY_STATE_ON,
-    SKY_STATE_OFF,
-    APP_EPG,
-    KNOWN_COUNTRIES,
 )
-from .const import TEST_CHANNEL_LIST
-from .classes.channelepg import ChannelEPG
-from .classes.channellist import ChannelList
-from .classes.channel import Channel
-from .classes.programme import RecordedProgramme
-from .classes.media import Media
-from .classes.device import Device
+from .const_test import TEST_CHANNEL_LIST
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -73,12 +74,15 @@ class SkyQRemote:
         self._overrideCountry = None
         self._epgCountryCode = None
         self._serialNumber = None
-        self._channel = None
         self._test_channel = None
+        # self._test = False
         self._port = port
         self._jsonport = jsonPort
         self._soapControlURL = None
+        self._channel = None
         self._lastEpg = None
+        self._programme = None
+        self._lastProgrammeEpg = None
         self._currentApp = APP_EPG
         self._channels = []
         self._error = False
@@ -167,10 +171,10 @@ class SkyQRemote:
 
         return media
 
-    def getEpgData(self, sid, epgDate, days=2, timeSpecific=False):
+    def getEpgData(self, sid, epgDate, days=2):
         """Get EPG data for the specified channel/date."""
-        epg = str(sid) + epgDate.strftime("%Y%m%d")
-        if self._lastEpg == epg and not timeSpecific:
+        epg = f"{str(sid)} {'{:0>2d}'.format(days)} {epgDate.strftime('%Y%m%d')}"
+        if self._lastEpg == epg:
             return self._channel
         self._lastEpg = epg
 
@@ -185,19 +189,14 @@ class SkyQRemote:
             channelName = channelNode["channel"]
             channelImageUrl = self._buildChannelUrl(sid, channelName)
 
-            if not timeSpecific:
-                for n in range(days):
-                    programmesData = self._remoteCountry.getEpgData(
-                        sid, channelNo, epgDate + timedelta(days=n)
-                    )
-                    if len(programmesData) > 0:
-                        programmes = programmes.union(programmesData)
-                    else:
-                        break
-            else:
-                programmes = self._remoteCountry.getTimeEpgData(
-                    sid, channelNo, epgDate, epgDate + timedelta(hours=12)
+            for n in range(days):
+                programmesData = self._remoteCountry.getEpgData(
+                    sid, channelNo, epgDate + timedelta(days=n)
                 )
+                if len(programmesData) > 0:
+                    programmes = programmes.union(programmesData)
+                else:
+                    break
 
         self._channel = ChannelEPG(
             sid, channelNo, channelName, channelImageUrl, sorted(programmes)
@@ -207,16 +206,14 @@ class SkyQRemote:
 
     def getProgrammeFromEpg(self, sid, epgDate, queryDate):
         """Get programme from EPG for specfied time and channel."""
-        epgData = self.getEpgData(sid, epgDate)
-
-        timechange = 0
-        while (
-            len(epgData.programmes) > 0 and queryDate < epgData.programmes[0].starttime
+        programmeEpg = f"{str(sid)} {epgDate.strftime('%Y%m%d')}"
+        if (
+            self._lastProgrammeEpg == programmeEpg
+            and queryDate < self._programme.endtime
         ):
-            timechange += 1
-            epgData = self.getEpgData(
-                sid, epgDate - timedelta(hours=timechange), timeSpecific=True
-            )
+            return self._programme
+
+        epgData = self.getEpgData(sid, epgDate)
 
         if len(epgData.programmes) == 0:
             if not self._error:
@@ -224,7 +221,7 @@ class SkyQRemote:
                 _LOGGER.info(
                     f"I0020 - Programme data not found for SID {sid} : {epgDate}"
                 )
-                return PAST_END_OF_EPG
+                return EPG_ERROR_NO_DATA
         else:
             self._error = False
 
@@ -234,26 +231,37 @@ class SkyQRemote:
                 for p in epgData.programmes
                 if p.starttime <= queryDate and p.endtime >= queryDate
             )
+
+            self._programme = programme
+            self._lastProgrammeEpg = programmeEpg
             return programme
 
         except StopIteration:
-            return PAST_END_OF_EPG
+            return EPG_ERROR_PAST_END
 
     def getCurrentLiveTVProgramme(self, sid):
         """Get current live programme on the specified channel."""
         try:
             queryDate = datetime.utcnow()
-            # queryDate = datetime.strptime(
-            #     "2020-06-05 00:15:27.243860", "%Y-%m-%d %H:%M:%S.%f"
-            # )
+            # seconds = queryDate.strftime("%S")
+            # if not self._test:
+            #     queryDate = datetime.strptime(
+            #         "2020-06-06 23:59:" + seconds + ".0000", "%Y-%m-%d %H:%M:%S.%f"
+            #     )
+            #     self._test = True
+            # else:
+            #     queryDate = datetime.strptime(
+            #         "2020-06-07 00:00:" + seconds + ".0000", "%Y-%m-%d %H:%M:%S.%f"
+            #     )
+            # print(f"{self._overrideCountry} - {queryDate}")
             programme = self.getProgrammeFromEpg(sid, queryDate, queryDate)
-            if programme is None:
-                return PAST_END_OF_EPG
+            if not isinstance(programme, Programme):
+                return None
 
             return programme
         except Exception as err:
             _LOGGER.exception(f"X0030 - Error occurred: {self._host} : {sid} : {err}")
-            return PAST_END_OF_EPG
+            return None
 
     def getRecording(self, pvrId):
         """Get the recording details."""
